@@ -84,36 +84,46 @@ pub fn register_custom_language(config_path: Option<PathBuf>) -> Result<()> {
   Ok(())
 }
 
+fn build_util_walker(base_dir: &Path, util_dirs: Option<Vec<PathBuf>>) -> Option<WalkBuilder> {
+  let mut util_dirs = util_dirs?.into_iter();
+  let first = util_dirs.next()?;
+  let mut walker = WalkBuilder::new(base_dir.join(first));
+  for dir in util_dirs {
+    walker.add(base_dir.join(dir));
+  }
+  Some(walker)
+}
+
 fn find_util_rules(
   base_dir: &Path,
   util_dirs: Option<Vec<PathBuf>>,
 ) -> Result<GlobalRules<SgLang>> {
-  let Some(util_dirs) = util_dirs else {
+  let Some(mut walker) = build_util_walker(base_dir, util_dirs) else {
     return Ok(GlobalRules::default());
   };
   let mut utils = vec![];
-  // TODO: use WalkBuilder::add to avoid loop
-  for dir in util_dirs {
-    let dir_path = base_dir.join(dir);
-    let walker = WalkBuilder::new(&dir_path)
-      .types(config_file_type())
-      .build();
-    for dir in walker {
-      let config_file = dir.with_context(|| EC::WalkRuleDir(dir_path.clone()))?;
-      // file_type is None only if it is stdin, safe to unwrap here
-      if !config_file
-        .file_type()
-        .expect("file type should be available for non-stdin")
-        .is_file()
-      {
-        continue;
-      }
-      let path = config_file.path();
-      let file = read_to_string(path)?;
-      let new_configs = from_str(&file)?;
-      utils.push(new_configs);
+  let walker = walker.types(config_file_type()).build();
+  for dir in walker {
+    let dir_path = dir
+      .as_ref()
+      .map(|entry| entry.path().to_path_buf())
+      .ok()
+      .unwrap();
+    let config_file = dir.with_context(|| EC::WalkRuleDir(dir_path.clone()))?;
+    // file_type is None only if it is stdin, safe to unwrap here
+    if !config_file
+      .file_type()
+      .expect("file type should be available for non-stdin")
+      .is_file()
+    {
+      continue;
     }
+    let path = config_file.path();
+    let file = read_to_string(path)?;
+    let new_configs = from_str(&file)?;
+    utils.push(new_configs);
   }
+
   let ret = DeserializeEnv::parse_global_utils(utils).context(EC::InvalidGlobalUtils)?;
   Ok(ret)
 }
