@@ -21,6 +21,11 @@ pub enum CaseStatus<'a> {
   Validated,
   /// Reported correct issue for invalid code
   Reported,
+  /// User accepted new snapshot updates
+  Updated {
+    source: &'a str,
+    updated: TestSnapshot,
+  },
   /// Reported issues for invalid code but it is wrong
   Wrong {
     source: &'a str,
@@ -75,6 +80,28 @@ impl<'a> CaseStatus<'a> {
       },
     }
   }
+
+  pub fn accept(&mut self) -> bool {
+    let CaseStatus::Wrong { source, actual, .. } = self else {
+      return false;
+    };
+    let updated = std::mem::replace(
+      actual,
+      TestSnapshot {
+        fixed: None,
+        labels: vec![],
+      },
+    );
+    *self = CaseStatus::Updated { source, updated };
+    true
+  }
+
+  pub fn is_pass(&self) -> bool {
+    matches!(
+      self,
+      CaseStatus::Validated | CaseStatus::Reported | CaseStatus::Updated { .. }
+    )
+  }
 }
 
 /// The result for one rule-test.yml
@@ -88,17 +115,14 @@ pub struct CaseResult<'a> {
 impl<'a> CaseResult<'a> {
   /// Did all cases in the rule-test pass the test?
   pub fn passed(&self) -> bool {
-    self
-      .cases
-      .iter()
-      .all(|c| matches!(c, CaseStatus::Validated | CaseStatus::Reported))
+    self.cases.iter().all(CaseStatus::is_pass)
   }
   pub fn changed_snapshots(&self) -> TestSnapshots {
     let snapshots = self
       .cases
       .iter()
       .filter_map(|c| match c {
-        CaseStatus::Wrong { source, actual, .. } => Some((source.to_string(), actual.clone())),
+        CaseStatus::Updated { source, updated } => Some((source.to_string(), updated.clone())),
         _ => None,
       })
       .collect();
