@@ -1,17 +1,21 @@
 use crate::GlobalRules;
 
-use crate::check_var::check_rewriters_in_transform;
+use crate::check_var::{check_rewriters_in_transform, CheckHint};
 use crate::fixer::Fixer;
 use crate::rule::DeserializeEnv;
-pub use crate::rule_core::{RuleConfigError, RuleCore, SerializableRuleCore};
+use crate::rule_core::{RuleCore, RuleCoreError, SerializableRuleCore};
+
 use ast_grep_core::language::Language;
 use ast_grep_core::replacer::Replacer;
 use ast_grep_core::{NodeMatch, StrDoc};
+
 use schemars::JsonSchema;
 use serde::{Deserialize, Serialize};
+use serde_yaml::Error as YamlError;
 use serde_yaml::{with::singleton_map_recursive::deserialize, Deserializer};
+use thiserror::Error;
 
-use std::collections::HashMap;
+use std::collections::{HashMap, HashSet};
 use std::ops::{Deref, DerefMut};
 
 #[derive(Serialize, Deserialize, Clone, Default, JsonSchema)]
@@ -30,6 +34,20 @@ pub enum Severity {
   Off,
 }
 
+#[derive(Debug, Error)]
+pub enum RuleConfigError {
+  #[error("Fail to parse yaml as RuleConfig")]
+  Yaml(#[from] YamlError),
+  #[error("Fail to parse yaml as Rule.")]
+  Core(#[from] RuleCoreError),
+  #[error("Rewriter rule `{1}` is not configured correctly.")]
+  Rewriter(#[source] RuleCoreError, String),
+  #[error("Undefined rewriter `{0}` used in transform.")]
+  UndefinedRewriter(String),
+  #[error("Rewriter rule `{0}` should have `fix`.")]
+  NoFixInRewriter(String),
+}
+
 #[derive(Serialize, Deserialize, Clone, JsonSchema)]
 pub struct SerializableRewriter {
   #[serde(flatten)]
@@ -82,11 +100,11 @@ impl<L: Language> SerializableRuleConfig<L> {
       .with_globals(globals)
       .with_rewriters(&rewriters);
     let rule = self.core.get_matcher(env)?;
-    self.register_rewriter(&rule, globals, &rewriters)?;
+    self.register_rewriters(&rule, globals, &rewriters)?;
     Ok(rule)
   }
 
-  fn register_rewriter(
+  fn register_rewriters(
     &self,
     rule: &RuleCore<L>,
     globals: &GlobalRules<L>,
@@ -97,18 +115,36 @@ impl<L: Language> SerializableRuleConfig<L> {
     };
     let vars = rule.defined_vars();
     for val in ser {
-      // NB should inherit env from matcher to inherit utils
-      // TODO: optimize duplicate env creation/util registration
-      let env = DeserializeEnv::new(self.language.clone())
-        .with_globals(globals)
-        .with_rewriters(rewriters);
-      let env = self.get_deserialize_env(env)?;
-      let rewriter = val.core.get_rewriter(env, &vars)?;
-      rewriters.insert(&val.id, rewriter).expect("should work");
+      if val.core.fix.is_none() {
+        return Err(RuleConfigError::NoFixInRewriter(val.id.clone()));
+      }
+      self
+        .register_one_rewriter(val, &vars, globals, rewriters)
+        .map_err(|e| RuleConfigError::Rewriter(e, val.id.clone()))?;
     }
     check_rewriters_in_transform(rule, rewriters)?;
     Ok(())
   }
+
+  fn register_one_rewriter(
+    &self,
+    val: &SerializableRewriter,
+    vars: &HashSet<&str>,
+    globals: &GlobalRules<L>,
+    rewriters: &GlobalRules<L>,
+  ) -> Result<(), RuleCoreError> {
+    // NB should inherit env from matcher to inherit utils
+    // TODO: optimize duplicate env creation/util registration
+    let env = DeserializeEnv::new(self.language.clone())
+      .with_globals(globals)
+      .with_rewriters(rewriters);
+    let env = self.get_deserialize_env(env)?;
+    let rewriter = val
+      .core
+      .get_matcher_with_hint(env, CheckHint::Rewriter(vars))?;
+    rewriters.insert(&val.id, rewriter).expect("should work");
+    Ok(())
+  }
 }
 
 impl<L: Language> Deref for SerializableRuleConfig<L> {
@@ -155,7 +191,7 @@ impl<L: Language> RuleConfig<L> {
   pub fn get_fixer(&self) -> Result<Option<Fixer<L>>, RuleConfigError> {
     if let Some(fix) = &self.fix {
       let env = self.matcher.get_env(self.language.clone());
-      let parsed = Fixer::parse(fix, &env, &self.transform)?;
+      let parsed = Fixer::parse(fix, &env, &self.transform).map_err(RuleCoreError::Fixer)?;
       Ok(Some(parsed))
     } else {
       Ok(None)
@@ -429,10 +465,27 @@ rewriters:
     ",
     )
     .expect("should parse");
-    let rule = RuleConfig::try_from(rule, &Default::default()).expect("work");
-    let grep = TypeScript::Tsx.ast_grep("a = 456");
-    let nm = grep.root().find(&rule.matcher);
-    assert!(nm.is_none());
+    let ret = RuleConfig::try_from(rule, &Default::default());
+    assert!(matches!(ret, Err(RuleConfigError::Core(_))));
+  }
+
+  #[test]
+  fn test_rewriter_should_have_fix() {
+    let rule: SerializableRuleConfig<TypeScript> = from_str(
+      r"
+id: test
+rule: {kind: number}
+language: Tsx
+rewriters:
+- id: wrong
+  rule: {matches: num}",
+    )
+    .expect("should parse");
+    let ret = RuleConfig::try_from(rule, &Default::default());
+    match ret {
+      Err(RuleConfigError::NoFixInRewriter(name)) => assert_eq!(name, "wrong"),
+      _ => panic!("unexpected error"),
+    }
   }
 
   #[test]
@@ -517,6 +570,26 @@ rewriters:
     );
     assert_eq!(undefined, "not-defined");
   }
+  #[test]
+  fn test_wrong_rewriter() {
+    let rule: SerializableRuleConfig<TypeScript> = from_str(
+      r"
+id: test
+rule: {pattern: 'a = $A'}
+language: Tsx
+rewriters:
+- id: wrong
+  rule: {kind: '114'}
+  fix: '1919810'
+    ",
+    )
+    .expect("should parse");
+    let ret = RuleConfig::try_from(rule, &Default::default());
+    match ret {
+      Err(RuleConfigError::Rewriter(_, name)) => assert_eq!(name, "wrong"),
+      _ => panic!("unexpected error"),
+    }
+  }
 
   #[test]
   fn test_undefined_rewriter_in_transform() {
@@ -545,7 +618,7 @@ id: test
 rule: {pattern: '$B = $A'}
 language: Tsx
 transform:
-  B: { rewrite: { rewriters: [re], source: $A } }
+  D: { rewrite: { rewriters: [re], source: $A } }
 rewriters:
 - id: re
   rule: {kind: number, pattern: $C}
